﻿///////////////////////////////////////////////////////////////////////////////
//
// (C) 2015 ICE TEA GROUP LLC - ALL RIGHTS RESERVED
//
// 
//
// ALL INFORMATION CONTAINED HEREIN IS, AND REMAINS
// THE PROPERTY OF ICE TEA GROUP LLC AND ITS SUPPLIERS, IF ANY.
// THE INTELLECTUAL PROPERTY AND TECHNICAL CONCEPTS CONTAINED
// HEREIN ARE PROPRIETARY TO ICE TEA GROUP LLC AND ITS SUPPLIERS
// AND MAY BE COVERED BY U.S. AND FOREIGN PATENTS, PATENT IN PROCESS, AND
// ARE PROTECTED BY TRADE SECRET OR COPYRIGHT LAW.
//
// DISSEMINATION OF THIS INFORMATION OR REPRODUCTION OF THIS MATERIAL
// IS STRICTLY FORBIDDEN UNLESS PRIOR WRITTEN PERMISSION IS OBTAINED
// FROM ICE TEA GROUP LLC.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using Wisej.Base;

namespace Wisej.Web
{
	/// <summary>
	/// The TreeViewComboBox control represents a <see cref="T:Wisej.Web.UserComboBox"/> control with a <see cref="T:Wisej.Web.ListView"/>
	/// as the drop down panel.
	/// </summary>
	[ToolboxItem(true)]
	[ToolboxBitmap(typeof(System.Windows.Forms.ListView))]
	[SRDescription("DescriptionListViewComboBox")]
	[DefaultEvent("SelectedItemChanged")]
	[DefaultProperty("Items")]
	[Designer("Wisej.Web.Design.ListViewComboBoxDesigner, Wisej.Web.Design")]
	public class ListViewComboBox : UserComboBox
	{
		#region Constructors

		/// <summary>
		/// Initializes a new TreeViewComboBox control.
		/// </summary>
		public ListViewComboBox()
		{
			this.DropDownControl = new ListView()
			{
				View = View.Details,
				MultiSelect = false,
				HeaderStyle = ColumnHeaderStyle.NonClickable
			};

			base.DropDownStyle = ComboBoxStyle.DropDownList;
		}

		#endregion

		#region Properties

		/// <summary>
		/// Returns the <see cref="T:Wisej.Web.ListView"/> control associated with this <see cref="T:Wisej.Web.ListViewComboBox"/>.
		/// </summary>
		[Browsable(false)]
		public ListView ListView
		{
			get { return (ListView)base.DropDownControl; }
		}

		/// <summary>
		/// Returns or sets the drop down control.
		/// </summary>
		[Browsable(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Control DropDownControl
		{
			get { return base.DropDownControl; }
			set
			{
				if (value == null)
					throw new ArgumentNullException("value");

				if (base.DropDownControl != value)
				{
					var oldListView = this.ListView;
					if (oldListView != null)
						oldListView.ItemSelectionChanged -= ListView_ItemSelectionChanged;

					var newListView = (ListView)value;
					newListView.Height = this.DropDownHeight;
					newListView.ItemSelectionChanged += ListView_ItemSelectionChanged;

					base.DropDownControl = value;
				}
			}
		}

		/// <summary>
		/// Returns or sets the selected item in the drop down <see cref="T:Wisej.Web.ListView"/> control.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new ListViewItem SelectedItem
		{
			get { return this.ListView.FocusedItem; }
			set { this.ListView.FocusedItem = value; }
		}

		/// <summary>
		/// Returns or sets the index of the currently selected item.
		/// </summary>
		/// <returns>A zero-based index of the currently selected item. -1 if no item is selected.</returns>
		/// <exception cref="T:System.ArgumentOutOfRangeException">The specified index is less than -1; o the specified index is greater than the number of items in the combo box. </exception>
		[Browsable(false)]
		[SRDescription("ComboBoxSelectedIndexDescr")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override int SelectedIndex
		{
			get
			{
				// return the index of the selected item.
				return this.ListView.SelectedIndex;
			}

			set
			{
				if (value < -1 || value > this.ListView.ItemCount)
					throw new ArgumentOutOfRangeException("SelectedIndex", SR.GetString("InvalidArgument", "SelectedIndex", value));

				if (value == -1)
					this.ListView.ClearSelection();
				else
					this.ListView.Items[value].Focused = true;
			}
		}

		/// <summary>
		/// Returns or sets the text associated with this control.
		/// </summary>
		/// <returns>The text associated with this control.</returns>
		[Bindable(false)]
		[Localizable(false)]
		[Browsable(false)]
		[DefaultValue("")]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Text
		{
			get { return this._text; }
			set
			{
				value = value ?? string.Empty;

				if (this._text != value)
				{
					this._text = value;
					Update();
				}
			}
		}
		private string _text = string.Empty;

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Watermark
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override object DataSource
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string DisplayMember
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string ValueMember
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override bool AllowHtml
		{
			get { return false; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new bool ReadOnly
		{
			get { return false; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override bool SpellCheck
		{
			get { return false; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override AutoCompleteMode AutoCompleteMode
		{
			get { return AutoCompleteMode.None; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(ComboBoxStyle.DropDownList)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override ComboBoxStyle DropDownStyle
		{
			get { return ComboBoxStyle.DropDownList; }
			set { }
		}

		/// <summary>
		/// Returns a collection containing all items in the control.
		/// </summary>
		/// <returns>A <see cref="T:Wisej.Web.ListView.ListViewItemCollection" /> that contains all the items in the <see cref="T:Wisej.Web.ListView" /> control.</returns>        
		[Localizable(true)]
		[MergableProperty(false)]
		[SRCategory("CatBehavior")]
		[SRDescription("ListViewItemsDescr")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public new ListView.ListViewItemCollection Items
		{
			get { return this.ListView.Items; }
		}

		/// <summary>
		/// Returns the collection of all column headers that appear in the control.
		/// </summary>
		/// <returns>A <see cref="T:Wisej.Web.ListView.ColumnHeaderCollection" /> that represents the column headers that appear when the <see cref="P:Wisej.Web.ListView.View" /> property is set to <see cref="F:Wisej.Web.View.Details" />.</returns>        
		[Localizable(true)]
		[MergableProperty(false)]
		[SRCategory("CatBehavior")]
		[SRDescription("ListViewColumnsDescr")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public ListView.ColumnHeaderCollection Columns
		{
			get { return this.ListView.Columns; }
		}

		/// <summary>
		/// Returns or sets the <see cref="T:Wisej.Web.ImageList" /> to use to display the main icon in the <see cref="T:Wisej.Web.ListViewItem"/> elements.
		/// </summary>
		/// <returns>An <see cref="T:Wisej.Web.ImageList" /> that contains the icons to use when the <see cref="P:Wisej.Web.ListView.View" /> property is set to <see cref="F:Wisej.Web.View.SmallIcon" />. The default is null.</returns>        
		[DefaultValue(null)]
		[SRCategory("CatBehavior")]
		[SRDescription("ListViewSmallImageListDescr")]
		public ImageList SmallImageList
		{
			get { return this.ListView.SmallImageList; }
			set { this.ListView.SmallImageList = value; }
		}

		/// <summary>
		/// Returns or sets the <see cref="T:Wisej.Web.ImageList" /> to use to display the main icon in the <see cref="T:Wisej.Web.ListViewItem"/> elements.
		/// </summary>
		/// <returns>An <see cref="T:Wisej.Web.ImageList" /> that contains the icons to use when the <see cref="P:Wisej.Web.ListView.View" /> property is set to <see cref="F:Wisej.Web.View.LargeIcon" />. The default is null.</returns>        
		[DefaultValue(null)]
		[SRCategory("CatBehavior")]
		[SRDescription("ListViewLargeImageListDescr")]
		public ImageList LargeImageList
		{
			get { return this.ListView.LargeImageList; }
			set { this.ListView.LargeImageList = value; }
		}

		/// <summary>
		/// Returns or sets how items are displayed in the control.
		/// </summary>
		/// <returns>One of the <see cref="T:Wisej.Web.View" /> values. The default is <see cref="F:Wisej.Web.View.LargeIcon" />.</returns>
		/// <exception cref="T:System.ComponentModel.InvalidEnumArgumentException">The value specified is not one of the <see cref="T:Wisej.Web.View" /> values. </exception>
		[DefaultValue(View.Details)]
		[SRCategory("CatAppearance")]
		[SRDescription("ListViewViewDescr")]
		public View View
		{
			get { return this.ListView.View; }
			set { this.ListView.View = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// This method is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[EditorBrowsable(EditorBrowsableState.Never)]
		protected override ComboBox.ObjectCollection CreateItemCollection()
		{
			return null;
		}

		private void ListView_ItemSelectionChanged(object sender, ItemSelectionChangedEventArgs e)
		{
			UpdateText();

			OnSelectedItemChanged(EventArgs.Empty);
			OnSelectedIndexChanged(EventArgs.Empty);
		}

		// Update the text to match the selected item.
		//
		private void UpdateText()
		{
			this.Text = this.ListView.FocusedItem?.Text ?? string.Empty;
		}

		#endregion

	}
}
