﻿///////////////////////////////////////////////////////////////////////////////
//
// (C) 2015 ICE TEA GROUP LLC - ALL RIGHTS RESERVED
//
// 
//
// ALL INFORMATION CONTAINED HEREIN IS, AND REMAINS
// THE PROPERTY OF ICE TEA GROUP LLC AND ITS SUPPLIERS, IF ANY.
// THE INTELLECTUAL PROPERTY AND TECHNICAL CONCEPTS CONTAINED
// HEREIN ARE PROPRIETARY TO ICE TEA GROUP LLC AND ITS SUPPLIERS
// AND MAY BE COVERED BY U.S. AND FOREIGN PATENTS, PATENT IN PROCESS, AND
// ARE PROTECTED BY TRADE SECRET OR COPYRIGHT LAW.
//
// DISSEMINATION OF THIS INFORMATION OR REPRODUCTION OF THIS MATERIAL
// IS STRICTLY FORBIDDEN UNLESS PRIOR WRITTEN PERMISSION IS OBTAINED
// FROM ICE TEA GROUP LLC.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using Wisej.Base;

namespace Wisej.Web
{
	/// <summary>
	/// The TreeViewComboBox control represents a <see cref="T:Wisej.Web.UserComboBox"/> control with a <see cref="T:Wisej.Web.TreeView"/>
	/// as the drop down panel.
	/// </summary>
	[ToolboxItem(true)]
	[ToolboxBitmap(typeof(System.Windows.Forms.TreeView))]
	[SRDescription("DescriptionTreeViewComboBox")]
	[DefaultEvent("SelectedItemChanged")]
	[DefaultProperty("Nodes")]
	[Designer("Wisej.Web.Design.TreeViewComboBoxDesigner, Wisej.Web.Design")]
	public class TreeViewComboBox : UserComboBox
	{
		#region Constructors

		/// <summary>
		/// Initializes a new TreeViewComboBox control.
		/// </summary>
		public TreeViewComboBox()
		{
			this.DropDownControl = new TreeView()
			{
				SelectionMode = SelectionMode.One
			};

			base.DropDownStyle = ComboBoxStyle.DropDownList;
		}

		#endregion

		#region Properties

		/// <summary>
		/// Returns the <see cref="T:Wisej.Web.TreeView"/> control associated with this <see cref="T:Wisej.Web.TreeViewComboBox"/>.
		/// </summary>
		[Browsable(false)]
		public TreeView TreeView
		{
			get { return (TreeView)base.DropDownControl; }
		}

		/// <summary>
		/// Returns or sets the drop down control.
		/// </summary>
		[Browsable(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override Control DropDownControl
		{
			get { return base.DropDownControl; }
			set
			{
				if (value == null)
					throw new ArgumentNullException("value");

				if (base.DropDownControl != value)
				{
					var oldTreeView = this.TreeView;
					if (oldTreeView != null)
						oldTreeView.AfterSelect -= TreeView_AfterSelect;

					var newTreeView = (TreeView)value;
					newTreeView.Height = this.DropDownHeight;
					newTreeView.AfterSelect += TreeView_AfterSelect;

					base.DropDownControl = value;
				}
			}
		}

		/// <summary>
		/// Returns or sets the selected node in the drop down <see cref="T:Wisej.Web.TreeView"/> control.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new TreeNode SelectedItem
		{
			get { return this.TreeView.SelectedNode; }
			set { this.TreeView.SelectedNode = value; }
		}

		/// <summary>
		/// Returns or sets the index of the currently selected item.
		/// </summary>
		/// <returns>A zero-based index of the currently selected item. -1 if no item is selected.</returns>
		/// <exception cref="T:System.ArgumentOutOfRangeException">The specified index is less than -1; o the specified index is greater than the number of items in the combo box. </exception>
		[Browsable(false)]
		[SRDescription("ComboBoxSelectedIndexDescr")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override int SelectedIndex
		{
			get
			{
				// return the index of the selected tree node.
				int index = -1;
				return GetSelectedNodeIndex(this.Nodes, this.SelectedItem, ref index);
			}

			set
			{
				if (value < -1)
					throw new ArgumentOutOfRangeException("SelectedIndex", SR.GetString("InvalidArgument", "SelectedIndex", value));

				int index = value;
				SetSelectedNodeByIndex(this.Nodes, ref index);
			}
		}

		private bool SetSelectedNodeByIndex(TreeNodeCollection nodes, ref int index)
		{
			if (index == -1)
			{
				this.TreeView.SelectedNode = null;
				return true;
			}

			foreach (TreeNode node in nodes)
			{
				if (index == 0)
				{
					this.TreeView.SelectedNode = node;
					return true;
				}

				index--;

				if (node.NodeCount > 0)
				{
					if (SetSelectedNodeByIndex(node.Nodes, ref index))
						return true;
				}
			}

			return false;
		}

		private int GetSelectedNodeIndex(TreeNodeCollection nodes, TreeNode selectedNode, ref int index)
		{
			if (selectedNode == null)
				return -1;

			foreach (TreeNode node in nodes)
			{
				if (node == selectedNode)
					return index;

				index++;

				if (node.NodeCount > 0)
				{
					if (GetSelectedNodeIndex(node.Nodes, selectedNode, ref index) > -1)
						return index;
				}
			}
			return -1;
		}

		/// <summary>
		/// Returns or sets the text associated with this control.
		/// </summary>
		/// <returns>The text associated with this control.</returns>
		[Bindable(false)]
		[Localizable(false)]
		[Browsable(false)]
		[DefaultValue("")]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Text
		{
			get { return this._text; }
			set
			{
				value = value ?? string.Empty;

				if (this._text != value)
				{
					this._text = value;
					Update();
				}
			}
		}
		private string _text = string.Empty;

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Watermark
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override object DataSource
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string DisplayMember
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(null)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string ValueMember
		{
			get { return null; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override bool AllowHtml
		{
			get { return false; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new bool ReadOnly
		{
			get { return false; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override bool SpellCheck
		{
			get { return false; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override AutoCompleteMode AutoCompleteMode
		{
			get { return AutoCompleteMode.None; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[DefaultValue(ComboBoxStyle.DropDownList)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override ComboBoxStyle DropDownStyle
		{
			get { return ComboBoxStyle.DropDownList; }
			set { }
		}

		/// <summary>
		/// This property is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[Browsable(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override ComboBox.ObjectCollection Items
		{
			get { return null; }
		}

		/// <summary>
		/// Returns the collection of tree nodes that are assigned to the tree view control.
		/// </summary>
		/// <returns>A <see cref="T:Wisej.Web.TreeNodeCollection" /> that represents the tree nodes assigned to the tree view control.</returns>
		[Localizable(true)]
		[MergableProperty(false)]
		[SRCategory("CatBehavior")]
		[SRDescription("TreeViewNodesDescr")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public TreeNodeCollection Nodes
		{
			get { return this.TreeView.Nodes; }
		}

		/// <summary>
		/// Returns or sets the <see cref="T:Wisej.Web.ImageList" /> that contains 
		/// the <see cref="T:System.Drawing.Image" /> objects that are used by the tree nodes.
		/// </summary>
		/// <returns>The <see cref="T:Wisej.Web.ImageList" /> that contains the <see cref="T:System.Drawing.Image" /> objects that are used by the tree nodes. The default value is null.</returns>
		[DefaultValue(null)]
		[RefreshProperties(RefreshProperties.Repaint)]
		[SRCategory("CatBehavior")]
		[SRDescription("TreeViewImageListDescr")]
		public ImageList ImageList
		{
			get { return this.TreeView.ImageList; }
			set { this.TreeView.ImageList = value; }
		}

		/// <summary>
		/// Returns or sets the image-list index value of the default image that is displayed by the tree nodes.
		/// </summary>
		/// <returns>A zero-based index that represents the position of an <see cref="T:System.Drawing.Image" /> in an <see cref="T:Wisej.Web.ImageList" />. The default is zero.</returns>
		/// <exception cref="T:System.ArgumentOutOfRangeException">The specified index is less than -1.</exception>
		[DefaultValue(-1)]
		[Localizable(true)]
		[RefreshProperties(RefreshProperties.Repaint)]
		[RelatedImageList("ImageList")]
		[SRCategory("CatBehavior")]
		[SRDescription("TreeViewImageIndexDescr")]
		[TypeConverter(typeof(ImageIndexConverter))]
		[Editor("Wisej.Web.Design.ImageIndexEditor, Wisej.Web.Design", typeof(UITypeEditor))]
		public int ImageIndex
		{
			get { return this.TreeView.ImageIndex; }
			set { this.TreeView.ImageIndex = value; }
		}

		/// <summary>
		/// Returns or sets the key of the default image for each node in the <see cref="T:Wisej.Web.TreeView" /> control 
		/// when it is in an unselected state.
		/// </summary>
		/// <returns>The key of the default image shown for each node <see cref="T:Wisej.Web.TreeView" /> control when the node is in an unselected state.</returns>
		[DefaultValue("")]
		[Localizable(true)]
		[RefreshProperties(RefreshProperties.Repaint)]
		[RelatedImageList("ImageList")]
		[SRCategory("CatBehavior")]
		[SRDescription("TreeViewImageKeyDescr")]
		[TypeConverter(typeof(ImageKeyConverter))]
		[Editor("Wisej.Web.Design.ImageIndexEditor, Wisej.Web.Design", typeof(UITypeEditor))]
		public string ImageKey
		{
			get { return this.TreeView.ImageKey; }
			set { this.TreeView.ImageKey = value; }
		}

		/// <summary>
		/// Returns or sets whether the tree nodes in the tree view are sorted.
		/// </summary>
		/// <returns>true if the tree nodes in the tree view are sorted; otherwise, false. The default is false.</returns>
		[Browsable(false)]
		[DefaultValue(false)]
		[EditorBrowsable(EditorBrowsableState.Never)]
		[SRCategory("CatBehavior")]
		[SRDescription("TreeViewSortedDescr")]
		public override bool Sorted
		{
			get { return this.TreeView.Sorted; }
			set { this.TreeView.Sorted = value; }
		}

		/// <summary>
		/// Returns or sets the implementation of <see cref="T:System.Collections.IComparer" /> to perform a 
		/// custom sort of the <see cref="T:Wisej.Web.TreeView" /> nodes.
		/// </summary>
		/// <returns>The <see cref="T:System.Collections.IComparer" /> to perform the custom sort.</returns>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[SRCategory("CatBehavior")]
		[SRDescription("TreeViewNodeSorterDescr")]
		public IComparer<TreeNode> TreeViewNodeSorter
		{
			get { return this.TreeView.TreeViewNodeSorter; }
			set { this.TreeView.TreeViewNodeSorter = value; }
		}

		#endregion

		#region Methods

		/// <summary>
		/// This method is not relevant for this control.
		/// </summary>
		/// <exclude/>
		[EditorBrowsable(EditorBrowsableState.Never)]
		protected override ComboBox.ObjectCollection CreateItemCollection()
		{
			return null;
		}

		private void TreeView_AfterSelect(object sender, TreeViewEventArgs e)
		{
			this.DroppedDown = false;

			UpdateText();
			OnSelectedItemChanged(EventArgs.Empty);
			OnSelectedIndexChanged(EventArgs.Empty);
		}

		// Update the text to match the selected item.
		//
		private void UpdateText()
		{
			this.Text = this.TreeView.SelectedNode?.Text ?? string.Empty;
		}

		#endregion

	}
}
